/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.view.admin.AddUsersDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.rational.report.web.core.client.internal.SecurityClient");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");

var CommonMessages	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
var SecurityClient	= com.ibm.rational.report.web.core.client.internal.SecurityClient;

/**
 * Displays a dialog to add users and team to grant access to a report design.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.view.admin.AddUsersDialog",
	[dijit._Widget, dijit._Templated],	{
	
	templatePath: dojo.moduleUrl("com.ibm.rational.report.web.core", "app/view/admin/templates/AddUsersDialog.html"),
	
	constructor: function(args) {
		this._controller = args.controller;
		this._title = args.title;
		this._resource = args.resource;
		this._principalMap = args.principalMap;
		this._allPrincipals = args.allPrincipals;
		this._aclEntryPrincipals = args.aclEntryPrincipals;
		this._postSave = args.postSave;
		this._postCancel = args.postCancel;
		this._parentWidget = args.parentWidget;
	},
	
	postCreate: function() {
		this._initText();
		
		this.populateContributers();
		
		this.searchUsers.value = CommonMessages.TYPE_TO_FILTER;
		this.searchUsers.disabled = false;
		this.okButton.disabled = true;
		
		var self = this;
				
		dojo.connect(this.searchUsers, "onfocus",
			function(e){
				self.searchUsers.value = "";
				self.searchButton.disabled = true;
		});
		
		dojo.connect(this.searchButton, "onclick",
			function(e){
				self._searchPattern();
				self.okButton.disabled = true;
		});
		
		dojo.connect(this.searchUsers, "onkeyup",
			function(e) {
				e.preventDefault();
				var name = dojo.trim(self.searchUsers.value);
				self.searchButton.disabled = (name.length == 0);
			}
		);
		
		this.dialog = new net.jazz.ajax.ui.internal.Dialog({
			contentNode: this.contentNode,
			primaryTitle: this._title,
			width: "500px",
			isClosable: true
		});		
	},
	
	_initText: function() {
		dojox.data.dom.removeChildren(this.permDialogText);
		this.permDialogText.appendChild(document.createTextNode(CommonMessages.ADD_USERS_SEARCH_TEXT));

		dojox.data.dom.removeChildren(this.okButton);
		this.okButton.appendChild(document.createTextNode(CommonMessages.OK_BUTTON_LABEL));
		
		dojox.data.dom.removeChildren(this.cancelButton);
		this.cancelButton.appendChild(document.createTextNode(CommonMessages.CLOSE_BUTTON_LABEL));
		
		dojox.data.dom.removeChildren(this.searchButton);
		this.searchButton.appendChild(document.createTextNode(CommonMessages.SEARCH_BUTTON_LABEL));
		
		dojox.data.dom.removeChildren(this.searchUsers);
		this.searchUsers.appendChild(document.createTextNode(CommonMessages.TYPE_TO_FILTER));
	},	

	_searchPattern: function(){
		var pattern = dojo.trim(this.searchUsers.value);
		var filter = "^";
		var matchingUserIds = new Array();
		var count = 0;
		var index = 0;

		// prefix with '^', replace * with .*, replace ? with ., add all other chars within []
		while (index < pattern.length) {
			var ch = pattern.substring(index, ++index);
			if (ch == "*") {
				filter = filter + ".*";
			}
			else if (ch == "?") {
				filter = filter + ".";
			}
			else {
				filter = filter + "[" + ch + "]" ;
			}
		}

		var regExp = new RegExp(filter);
		
		for (var index = 0; index < this._allPrincipals.length; index++) {
			var user = this._allPrincipals[index];
        	if (regExp.test(user.name)) {
    			matchingUserIds[count] = user.name; // Pattern found
    			count++;
			}
		}

    	this._deleteUserTable();
 
		// Add the users matching the pattern
		for (; count > 0; ) {
			count--;
			this._addOption(matchingUserIds[count]);
		}
	},

    _deleteUserTable: function(){
		var rows = this.usersTable.tBodies[0].getElementsByTagName("tr");
		var i =0;

		while(this.usersTable.tBodies[0].getElementsByTagName("tr").length > 0){
			if (this.usersTable.tBodies[0].getElementsByTagName("tr").length > 0) {
				this.usersTable.tBodies[0].deleteRow(0);
			}
		}
	},
	
	addContributers: function() {		
		var newPrincipals = [];
	
		var inputs = this.usersTableBody.getElementsByTagName("input");
		var userNames = new Array();
		dojo.forEach(inputs, function(input){
			if (input.checked) {
				newPrincipals.push(input.id);
			}
		});
			
		if (newPrincipals != null && newPrincipals.length > 0) {
			var permissionIds = new Array();
			var srh = new ServiceResponseHandler(this, "_postNewAclEntriesSuccess", "_postNewAclEntriesError");
			var params = {type: this._resource.type, repositoryItemId: this._resource.itemId,
						  principalIds: newPrincipals, permissionIds: permissionIds, negative: false};
			SecurityClient.postNewAclEntries(srh, params);
		}
		else {
			this._postNewAclEntriesSuccess();
		}
	},
	
	populateContributers: function() {
		var users = [];
		for(var i = 0 ; i < this._allPrincipals.length; i++){
			var principal = this._allPrincipals[i];
			var hasPermission = false;
			for(var j = 0 ; j<this._aclEntryPrincipals.length ; j++){
				if(principal.name == this._aclEntryPrincipals[j].name) {
					hasPermission = true;
					break;
				}
			}
			if(!hasPermission){
				users.push(principal);
			}
		}		
		for (var i = 0; i < users.length; i++) {
			var data = users[i].name;
			this._addOption(data);
		}
	},	

	_addOption: function(userName){
		var self = this;
		var row = document.createElement("tr");
		this.usersTableBody.appendChild(row);
		var td = row.insertCell(0);
		var input = document.createElement("input");
		input.type = "checkbox";
        input.onclick = function(){
            var result = true;
            var inputs = self.usersTableBody.getElementsByTagName("input");
            dojo.forEach(inputs, function(input){
                if (input.checked) {
                    result = false;
                }
            });
            self.okButton.disabled = result;
        };
        
        input.id = userName;
		td.appendChild(input);
		td = row.insertCell(1);
		var text = document.createTextNode(userName);
		td.appendChild(text);
	},
	
	_postNewAclEntriesSuccess : function() {
		this._parentWidget._refreshTable();
	},
	
	_postNewAclEntriesError : function() {	
	}
	
});

})();